@@ -1,48 +1,65 @@
 package Zoidberg::DispatchTable;
 
-our $VERSION = '0.41';
+our $VERSION = '0.42';
 
 use strict;
-use Carp;
-use Zoidberg::Utils qw/debug/;
-use Exporter::Tidy all => [qw/stack wipe/];
+use Zoidberg::Utils qw/debug bug error/;
+use Exporter::Tidy all => [qw/stack wipe tags/];
+
+#our $ERROR_CALLER = 1; 
 
 # $self->[0] hash with arrays of dispatch strings/refs
 # $self->[1] hash with arrays of tags
 # $self->[2] object ref
 # $self->[3] object can parent bit
 # $self->[4] array with keys to keep them in order
+# $self->[5] iteration index for keys()
+
+# keys are kept in order to avoid inconsistencies
+# for example when iterating trough {contexts}
+
+# ############# #
+# Tie interface #
+# ############# #
 
 sub TIEHASH  {
 	my $class = shift;
-	my $ref = shift || croak 'need object ref to tie hash';
+	my $ref = shift || error 'need object ref to tie hash';
 	# $ref is either array ref or object ref
 	my $self = (ref($ref) eq 'ARRAY')
 		? $ref
-		: [{}, {}, $ref, $ref->can('parent'), []];
+		: [{}, {}, $ref, $ref->can('parent'), [], 0];
 	bless $self, $class;
 	for my $ref (@_) { $self->STORE($_, $$ref{$_}) for keys %$ref }
 	return $self;
 }
 
-sub STORE { 
+sub STORE {
 	my ($self, $key, $value) = @_;
 	my $tag = 'undef';
 	($value, $tag) = @$value if ref($value) eq 'ARRAY';
-	
-	if (ref($value) eq 'HASH') {
-		unless (tied $value) {
+
+	my $t = ref $value;
+	if ($t eq 'HASH') {
+		unless (tied $value) { # recurs tie'ing
 			my %value;
 			tie %value, __PACKAGE__, $$self[2], $value;
 			$value = \%value;
 		}
+		# else just store the tied hash
 	}
-	elsif (ref($value) ne 'CODE') {
+	elsif ($t eq 'CODE') { # curry code ref
+		my $closure = $value; # strange error without copy
+		$value = sub { $closure->($$self[2], @_) };
+	}
+	elsif ($t) { bug "Can't store ref of type $t in DispatchTable" }
+	else {
 		$value =~ s/(^\s*|\s*$)//g;
-		croak "Can't use ==>$value<== as subroutine."
-			if ! length($value)
+		error "Can't use ==>$value<== as subroutine."
+			if ! length $value
 			or $value =~ /^\$/; # no vars
 	}
+
 	push @{$self->[0]{$key}}, $value;
 	push @{$self->[1]{$key}}, $tag;
 	push @{$self->[4]}, $key;
@@ -52,8 +69,7 @@ sub FETCH {
 	my ($self, $key) = @_;
 	if ($self->EXISTS($key)) {
 		$self->[0]{$key}[-1] = $self->convert($self->[0]{$key}[-1])
-			unless ref($self->[0]{$key}[-1]) eq 'CODE'
-			or     ref($self->[0]{$key}[-1]) eq 'HASH';
+			unless ref $self->[0]{$key}[-1];
 		return $self->[0]{$key}[-1];
 	}
 	elsif ($self->EXISTS('_AUTOLOAD')) {
@@ -71,17 +87,24 @@ sub FETCH {
 sub convert {
 	my ($self, $ding) = @_;
 
+	if ($ding =~ /^\s*sub\s*{.*}\s*$/) { # undocumented hack
+		debug "going to eval: $ding";
+		my $closure = eval $ding;
+		die if $@;
+		return sub { $closure->($$self[2], @_) };
+	}
+
         $ding =~ s#^->((\w+)->)?#
 		( $self->[3] ? q/parent->/         : '' ) .
 		( $1         ? qq/{objects}{$2}->/ : '' )
 	#e;
 
-	if ($ding =~ /\(\s*\)$/) { $ding =~ s/\s*\)$/\@_\)/ }
-        elsif ($ding =~ /\(.*\)$/) { $ding =~ s/\)$/, \@_\)/ }
+	if ($ding =~ /\(\s*\)$/s) { $ding =~ s/\s*\)$/\@_\)/ }
+        elsif ($ding =~ /\(.*\)$/s) { $ding =~ s/\)$/, \@_\)/ }
 	else { $ding .= '(@_)' }
 
-	debug "going to eval: ==>sub { \$self->[2]->$ding }<==\n";
-	my $sub = eval("sub { \$self->[2]->$ding }");
+	debug "going to eval: sub { \$self->[2]->$ding }";
+	my $sub = eval "sub { \$\$self[2]->$ding }";
 	die if $@;
 	return $sub;
 }
@@ -108,41 +131,48 @@ sub CLEAR    {
 	%{$_[0][0]} = ();
 	%{$_[0][1]} = ();
 	@{$_[0][4]} = ();
+	  $_[0][5]  =  0;
 }
 
-our $Iter = 0;
-
 sub FIRSTKEY {
-	$Iter = 0;
+	$_[0][5] = 0;
 	goto \&NEXTKEY
 }
 
 sub NEXTKEY  {
 	my $self = shift;
-	if ($Iter > scalar @{$self->[4]}) {
-		$Iter = 0;
+	if ($$self[5] > $#{$$self[4]}) {
+		$$self[5] = 0;
 		return wantarray ? () : undef;
 	}
-	elsif (wantarray) {
-		my $key = $self->[4][$Iter++];
+	elsif (wantarray) { # ($key, $value) = each(%table)
+		my $key = $$self[4][$$self[5]++];
 		return $key, $self->FETCH($key);
 	}
-	else { return $self->[4][$Iter++] }
+	else { return $self->[4][$$self[5]++] } # for $key (keys %table)
 }
 
+# ################# #
+# Exported routines #
+# ################# #
+
 sub stack {
 	my ($table, $key, $use_tag) = @_;
 	my $self = tied %$table;
-	for (@{$self->[0]{$key}}) {
-		$_ = $self->convert($_)
-			unless ref($_) eq 'CODE'
-			or     ref($_) eq 'HASH';
-	}
+	return undef unless exists $$self[0]{$key};
+	for (@{$self->[0]{$key}}) { $_ = $self->convert($_) unless ref $_ }
 	return map [ $$self[0]{$key}[$_], $$self[1]{$key}[$_] ], (0..$#{$$self[0]{$key}})
 		if $use_tag;
 	return @{$self->[0]{$key}};
 }
 
+sub tags {
+	my ($table, $key) = @_;
+	my $self = tied %$table;
+	return undef unless exists $$self[1]{$key};
+	return @{$self->[1]{$key}};
+}
+
 sub wipe {
 	my ($table, $tag, @keys) = @_;
 	my $self = tied %$table;
@@ -166,43 +196,6 @@ sub wipe {
 	return \%old;
 }
 
-package Zoidberg::DispatchScalar;
-
-use Carp;
-our @ISA = qw/Zoidberg::DispatchTable/;
-
-sub TIESCALAR {
-	my $class = shift;
-        my $ref = shift || croak 'need object ref to tie hash';
-        # $ref is either array ref or object ref
-        my $self = (ref($ref) eq 'ARRAY')
-                ? $ref
-                : [undef, undef, $ref, $ref->can('parent')];
-        bless $self, $class;
-        $self->STORE($_) for @_;
-        return $self;
-}
-
-sub STORE {
-	my ($self, $value) = @_;
-	if (ref($value) ne 'CODE') {
-		$value =~ s/(^\s*|\s*$)//g;
-		croak "Can't use ==>$value<== as subroutine."
-			if ! length($value)
-			or $value =~ /^\$/; # no vars
-	}
-	$$self[0] = $value;
-}
-
-sub FETCH {
-	my $self = shift;
-	return sub {} unless $$self[0];
-
-	$$self[0] = $self->convert($$self[0])
-		unless ref($$self[0]) eq 'CODE';
-	return $$self[0];
-}
-
 1;
 
 __END__
@@ -254,15 +247,43 @@ The following strings are supported:
 
 You can store either config strings or CODE references in the table.
 
+If you store a CODE reference it will be wrapped in order to make it look like a 
+method of the object to which the table belongs. The method will get an object
+reference as it's first argument.
+
 If you store an ARRAY ref it is expected to be of the form C<[$value, $tag]>,
 where C<$tag> is an identifier used for handling selections of the table.
 If you store a HASH ref it will be tied recursively as a DispatchTable.
 
-Also keys are kept in the order they are first added, and for each key a stack
-is used. Deleting a key only pops it's stack.
+Keys are kept in the order they are first added, thus C<keys(%table)> will always
+return the same order. This is to keep zoid's plugins in the order they are added.
+Also for each key a stack is used. Deleting a key only pops it's stack.
 
 I< This modules doesn't check for security issues, it just runs arbitrary code. >
 
+=head1 EXPORT
+
+This module can export the methods C<wipe>, C<stack> and C<tags>.
+
+=over 4
+
+=item C<wipe(\%table, $tag, @keys)>
+
+Wipes entries with tag C<$tag> from given set of kaeys or from the whole
+table.
+
+=item C<stack(\%table, $key, $tags)>
+
+Returns the whole stack for an given key, useful to loop trough stacks.
+C<$tags> is a boolean, when true all items are returned as a sub array of CODE ref
+with tag.
+
+=item C<tags(\%table, $key)>
+
+Returns an array of all tags for given key.
+
+=back
+
 =head1 AUTHOR
 
 Jaap Karssenberg || Pardus [Larus] E<lt>pardus@cpan.orgE<gt>
