@@ -1,6 +1,6 @@
 package Zoidberg::Fish::Buffer::Meta::Vim;
 
-our $VERSION = '0.41';
+our $VERSION = '0.42';
 
 use strict;
 use Zoidberg::Utils qw/debug/;
@@ -11,6 +11,7 @@ sub _switch_on { $_[0]->{_vim_meta_cmd} = '' }
 sub _do_key {
 	my ($self, $key) = @_;
 	if ($key eq 'esc') { $self->{_vim_meta_cmd} = '' }
+	elsif ($key eq 'return') { $self->{_vim_meta_cmd} = ''; $self->submit }
 	elsif (length($key) > 1) { $self->bell }
 	else {
 		$self->{_vim_meta_cmd} .= $key;
@@ -42,8 +43,14 @@ sub _try_it { # / <count>? ( <cmd> (<cmd> | <count>? <motion>) | <motion> /
 	return 1;
 }
 
-sub delete { print "\ndeleting stuff\n" }
+sub delete {
+	my $self = shift;
+	print "\ndeleting @_\n";
+}
 
-sub replace { print "\nreplacing stuff\n" }
+sub replace {
+	my $self = shift;
+	print "\nreplacing @_\n";
+}
 
 1;
